/* @flow */

import React, { Component, PropTypes } from 'react';
import {
  Editor,
  EditorState,
  ContentState,
  RichUtils,
  convertFromRaw,
  convertToRaw,
  CompositeDecorator,
  DefaultDraftBlockRenderMap,
} from 'draft-js';
import {
  changeDepth,
  handleNewLine,
  customStyleMap,
  getEntityOfSelection,
  getCurrentBlockType,
  getInlineStylesOfSelection,
  toggleInlineStyle,
  alignmentBlockStyleFn,
} from '../../utils/Editor';
import { Map } from 'immutable';
import InlineControl from '../InlineControl';
import BlockControl from '../BlockControl';
import FontSizeControl from '../FontSizeControl';
import ListControl from '../ListControl';
import TextAlignControl from '../TextAlignControl';
import ColorPicker from '../ColorPicker';
import LinkControl from '../LinkControl';
import ImageControl from '../ImageControl';
import HistoryControl from '../HistoryControl';
import LinkDecorator from '../../Decorators/Link';
import BlockRenderer from '../../Renderer';
import styles from './styles.css';
import draft from '../../../../css/Draft.css'; // eslint-disable-line no-unused-vars
import common from '../../../../css/common.css'; // eslint-disable-line no-unused-vars

export default class WysiwygEditor extends Component {

  static propTypes = {
    onChange: PropTypes.func,
    rawContent: PropTypes.object,
    toolbarAlwaysVisible: PropTypes.bool,
    toolbarClassName: PropTypes.string,
    editorClassName: PropTypes.string,
    wrapperClassName: PropTypes.string,
    placeholder: PropTypes.string,
    inlineControlInDropdown: PropTypes.bool,
    listControlInDropdown: PropTypes.bool,
    textAlignControlInDropdown: PropTypes.bool,
  };

  static defaultProps = {
    toolbarAlwaysVisible: false,
  };

  state: Object = {
    editorState: undefined,
    currentBlockType: 'unstyled',
    currentInlineStyles: {},
    toolBarMouseDown: false,
    editorFocused: false,
    editorMouseDown: false,
  };

  componentWillMount() {
    let editorState;
    const decorator = new CompositeDecorator([LinkDecorator]);
    if (this.props.rawContent) {
      editorState = EditorState.createWithContent(
        ContentState.createFromBlockArray(convertFromRaw(this.props.rawContent), decorator));
    } else {
      editorState = EditorState.createEmpty(decorator);
    }
    editorState = EditorState.moveFocusToEnd(editorState);
    this.setState({
      editorState,
    });
  }

  customBlockRenderMap: Map = DefaultDraftBlockRenderMap
  .merge(new Map({
    unstyled: {
      element: 'p',
    },
  }));

  _onChange: Function = (editorState: Object, afterChange: Function) => {
    const currentBlockType = getCurrentBlockType(editorState);
    const currentEntity = getEntityOfSelection(editorState);
    const currentInlineStyles = getInlineStylesOfSelection(editorState);
    this.setState({
      editorState,
      currentEntity,
      currentBlockType,
      currentInlineStyles,
    }, this._afterChange(afterChange));
  };

  _afterChange: Function = (afterChange) => {
    setTimeout(() => {
      if (afterChange) {
        afterChange();
      }
      if (this.props.onChange) {
        const editorContent = convertToRaw(this.state.editorState.getCurrentContent());
        this.props.onChange(editorContent);
      }
    });
  };

  _onToolbarMouseDown: Function = () => {
    this.setState({
      toolBarMouseDown: true,
    });
  };

  _onToolbarMouseUp: Function = () => {
    this.setState({
      toolBarMouseDown: false,
      editorFocused: true,
    });
  };

  _onEditorFocus:Function = () => {
    this.setState({
      toolBarMouseDown: false,
      editorFocused: true,
    });
  };

  _onEditorBlur: Function = () => {
    const { editorState } = this.state;
    const currentBlockType = getCurrentBlockType(editorState);
    const currentEntity = getEntityOfSelection(editorState);
    const currentInlineStyles = getInlineStylesOfSelection(editorState);
    this.setState({
      editorFocused: false,
      currentBlockType,
      currentEntity,
      currentInlineStyles,
    });
  };

  _onEditorMouseDown: Function = () => {
    this.setState({
      editorMouseDown: true,
    });
  };

  _onEditorMouseUp: Function = () => {
    this.setState({
      editorMouseDown: false,
    });
  };

  _focusEditor: Function = () => {
    this.refs.editor.focus();
  };

  _toggleInlineStyle: Function = (style) => {
    const newState = RichUtils.toggleInlineStyle(
      this.state.editorState,
      style
    );
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _toggleBlockType: Function = (blockType) => {
    const newState = RichUtils.toggleBlockType(
      this.state.editorState,
      blockType
    );
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _toggleColor: Function = (color) => {
    const newState = toggleInlineStyle(
      this.state.editorState,
      'color',
      color
    );
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _toggleFont: Function = (color) => {
    const newState = toggleInlineStyle(
      this.state.editorState,
      'fontSize',
      color
    );
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _undo: Function = () => {
    const newState = EditorState.undo(this.state.editorState);
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _redo: Function = () => {
    const newState = EditorState.redo(this.state.editorState);
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _handleKeyCommand: Function = (command): boolean => {
    const { editorState } = this.state;
    const newState = RichUtils.handleKeyCommand(editorState, command);
    if (newState) {
      this._onChange(newState, this._focusEditor);
      return true;
    }
    return false;
  };

  _onTab: Function = (event): boolean => {
    event.preventDefault();
    const editorState = changeDepth(this.state.editorState, event.shiftKey ? -1 : 1, 4);
    if (editorState) {
      this._onChange(editorState);
      return true;
    }
    return false;
  };

  _adjustDepth: Function = (adjustment): void => {
    const newState = changeDepth(
      this.state.editorState,
      adjustment,
      4,
    );
    if (newState) {
      this._onChange(newState, this._focusEditor);
    }
  };

  _handleReturn: Function = (event): boolean => {
    const editorState = handleNewLine(this.state.editorState, event);
    if (editorState) {
      this._onChange(editorState);
      return true;
    }
    return false;
  };

  render() {
    const {
      editorState,
      editorFocused,
      editorMouseDown,
      toolBarMouseDown,
      currentInlineStyles,
      currentBlockType,
      currentEntity,
     } = this.state;

    const {
      toolbarAlwaysVisible,
      inlineControlInDropdown,
      listControlInDropdown,
      textAlignControlInDropdown,
      placeholder,
      toolbarClassName,
      editorClassName,
      wrapperClassName,
    } = this.props;
    const hasFocus = editorFocused || toolBarMouseDown || editorMouseDown;

    return (
      <div className={`${styles.editor_wrapper} ${wrapperClassName}`}>
      {
        (hasFocus || toolbarAlwaysVisible) ?
          <div
            className={`${styles.editor_toolbar} ${toolbarClassName}`}
            onMouseDown={this._onToolbarMouseDown}
            onMouseUp={this._onToolbarMouseUp}
          >
            <InlineControl
              onToggle={this._toggleInlineStyle}
              currentInlineStyles={currentInlineStyles}
              inlineControlInDropdown={inlineControlInDropdown}
            />
            <BlockControl
              onToggle={this._toggleBlockType}
              currentBlockType={this.state.currentBlockType}
            />
            <FontSizeControl
              onToggle={this._toggleFont}
              currentFontSize={currentInlineStyles.FONTSIZE}
            />
            <ListControl
              onDepthChange={this._adjustDepth}
              onBlockTypeToggle={this._toggleBlockType}
              currentBlockType={currentBlockType}
              listControlInDropdown={listControlInDropdown}
            />
            <TextAlignControl
              editorState={editorState}
              onChange={this._onChange}
              controlInDropdown={textAlignControlInDropdown}
            />
            <ColorPicker
              onToggle={this._toggleColor}
              currentColor={currentInlineStyles.COLOR}
            />
            <LinkControl
              editorState={editorState}
              entity={currentEntity}
              onChange={this._onChange}
              hideModal={editorMouseDown || !hasFocus}
            />
            <ImageControl
              editorState={editorState}
              onChange={this._onChange}
              hideModal={editorMouseDown || !hasFocus}
            />
            <HistoryControl
              undo={this._undo}
              redo={this._redo}
              undoDisabled={editorState.getUndoStack().size === 0}
              redoDisabled={editorState.getRedoStack().size === 0}
            />
          </div>
        :
        undefined
      }
        <div
          className={`${styles.editor_main} ${editorClassName}`}
          onClick={this._focusEditor}
          onFocus={this._onEditorFocus}
          onBlur={this._onEditorBlur}
          onMouseUp={this._onEditorMouseUp}
          onMouseDown={this._onEditorMouseDown}
        >
          <Editor
            ref="editor"
            spellCheck
            onTab={this._onTab}
            editorState={editorState}
            onChange={this._onChange}
            placeholder={placeholder}
            blockStyleFn={alignmentBlockStyleFn}
            customStyleMap={customStyleMap}
            handleReturn={this._handleReturn}
            blockRendererFn={BlockRenderer}
            blockRenderMap={this.customBlockRenderMap}
            handleKeyCommand={this._handleKeyCommand}
          />
        </div>
      </div>
    );
  }
}

// todo: increase test coverage
// TODO: move funcs to respective components
// blur after toolbar to somewhere else should hide it
// todo: close modal on editor click
// toolbar does not go away on click outside
// todo: ESC key handling on editor to close modals
