/* @flow */

import React, { Component, PropTypes } from 'react';
import { Entity, AtomicBlockUtils } from 'draft-js';
import image from '../../../../images/image.svg';
import classNames from 'classnames';
import Option from '../Option';
import Spinner from '../Spinner';
import styles from './styles.css';

export default class ImageControl extends Component {

  static propTypes = {
    editorState: PropTypes.object.isRequired,
    onChange: PropTypes.func.isRequired,
    hideModal: PropTypes.bool,
  };

  state: Object = {
    showModal: false,
    imgSrc: '',
    showImageUpload: true,
    showImageLoading: false,
  };

  componentWillReceiveProps(properties: Object): void {
    if (properties.hideModal && this.state.showModal) {
      this.setState({
        showModal: false,
      });
    }
  }

  _showImageUploadOption: Function = () => {
    this.setState({
      showImageUpload: true,
    });
  };

  _showImageURLOption: Function = () => {
    this.setState({
      showImageUpload: false,
    });
  };

  _toggleShowImageLoading: Function = () => {
    const showImageLoading = !this.state.showImageLoading;
    this.setState({
      showImageLoading,
    });
  };

  _stopPropagation: Function = (event) => {
    event.preventDefault();
    event.stopPropagation();
  };

  _addImage: Function = (event, imgSrc): void => {
    const { editorState, onChange } = this.props;
    const src = imgSrc || this.state.imgSrc;

    const entityKey = Entity.create('IMAGE', 'MUTABLE', { src, width: 300 });
    const newEditorState = AtomicBlockUtils.insertAtomicBlock(
      editorState,
      entityKey,
      ' '
    );
    onChange(newEditorState);
    this._toggleModal();
  };

  _uploadImage: Function = (file: Object): void => {
    this._toggleShowImageLoading();
    const data = new FormData;
    data.append('image', file);
    const xhr = new XMLHttpRequest;
    xhr.open('POST', 'https://api.imgur.com/3/image');
    xhr.setRequestHeader('Authorization', 'Client-ID 2e4ea711654893b');
    xhr.send(data);
    xhr.addEventListener('load', () => {
      this._toggleShowImageLoading();
      const response = JSON.parse(xhr.responseText);
      this._addImage(undefined, response.data.link);
    });
  };

  _selectImage: Function = (event: Object): void => {
    if (event.target.files && event.target.files.length > 0) {
      this._uploadImage(event.target.files[0]);
    }
  };

  _onImageDrop: Function = (event) => {
    event.preventDefault();
    event.stopPropagation();
    this._uploadImage(event.dataTransfer.files[0]);
  };

  _updateImageSrc: Function = (event: Object): void => {
    this.setState({
      imgSrc: event.target.value,
    });
  };

  _toggleModal: Function = (): void => {
    const { showModal } = this.state;
    const newState = {};
    newState.showModal = !showModal;
    newState.imgSrc = undefined;
    this.setState(newState);
  };

  renderAddLinkModal() {
    const { imgSrc, showImageUpload, showImageLoading } = this.state;
    return (
      <div className={styles['image-modal']}>
        <div className={styles['image-modal-dialog']}>
          <div className={styles['image-modal-header']}>
            <span
              onClick={this._showImageUploadOption}
              className={styles.image_file_option}
            >
              <span>File Upload</span>
              <span
                className={classNames(
                  styles['image-modal-header-text'],
                  { [`${styles['image-modal-header-text-highlighted']}`]: showImageUpload }
                )}
              />
            </span>
            <span
              onClick={this._showImageURLOption}
              className={styles.image_file_option}
            >
              <span>URL</span>
              <span
                className={classNames(
                  styles['image-modal-header-text'],
                  { [`${styles['image-modal-header-text-highlighted']}`]: !showImageUpload }
                )}
              />
            </span>
          </div>
          <div className={styles['image-modal-content']}>
          {
            showImageUpload ?
              <div>
                <div
                  onDragEnter={this._stopPropagation}
                  onDragOver={this._stopPropagation}
                  onDrop={this._onImageDrop}
                  className={styles.image_upload}
                >
                  <label
                    htmlFor="file"
                    className={styles.image_upload_label}
                  >
                    Drop the file or click to upload
                  </label>
                </div>
                <input
                  type="file"
                  id="file"
                  onChange={this._selectImage}
                  className={styles.image_modal_fileupload}
                />
              </div> :
              <div
                className={styles.image_url_section}
              >
                <input
                  className={styles.image_modal_imageURLinput}
                  placeholder="Enter url"
                  onChange={this._updateImageSrc}
                  onBlur={this._updateImageSrc}
                  value={imgSrc}
                />
              </div>
          }
            {showImageLoading ?
              <div className={styles.image_modal_spinner}>
                <Spinner />
              </div> :
              undefined}
          </div>
          <div className={styles['image-modal-footer']}>
            <button
              className={styles.image_modal_btn}
              onClick={this._addImage}
              disabled={!imgSrc}
            >
              Add
            </button>
            <button
              className={styles.image_modal_btn}
              onClick={this._toggleModal}
            >
              Cancel
            </button>
          </div>
        </div>
      </div>
    );
  }

  render(): Object {
    const { showModal } = this.state;
    return (
      <div className={styles.image_wrapper}>
        <Option
          value="unordered-list-item"
          onClick={this._toggleModal}
        >
          <a>
            <img
              src={image}
              role="presentation"
            />
          </a>
        </Option>
        {showModal ? this.renderAddLinkModal() : undefined}
      </div>
    );
  }
}

// should also close when clicked anywhere else on page after toolbar click
// Highlight image icon when image is seleted
// todo: fix class names
// todo: reset highlight as modal opens
// todo: image selection should highlight option
