/* @flow */

import React, { Component, PropTypes } from 'react';
import styles from './styles.css';
import Option from '../Option';
import { Entity, RichUtils, EditorState, Modifier } from 'draft-js';
import { getCurrentSelectionText, getEntityRange } from '../../utils/Editor';
import link from '../../../../images/link.svg';
import unlink from '../../../../images/unlink.svg';

export default class LinkControl extends Component {

  static propTypes = {
    editorState: PropTypes.object.isRequired,
    onChange: PropTypes.func.isRequired,
    hideModal: PropTypes.bool,
    entity: PropTypes.string,
  };

  state: Object = {
    showLinkModal: false,
    linkTarget: '',
    linkTitle: '',
  };

  componentWillReceiveProps(properties: Object): void {
    if (properties.hideModal && this.state.showLinkModal) {
      this.setState({
        showLinkModal: false,
      });
    }
  }

  _toggleLinkModal: Function = (): void => {
    const { editorState, entity } = this.props;
    const { showLinkModal } = this.state;
    const newState = {};
    newState.showLinkModal = !showLinkModal;
    if (newState.showLinkModal) {
      newState.entity = entity;
      const entityRange = entity && getEntityRange(editorState, entity);
      newState.linkTarget = entity && Entity.get(entity).get('data').url;
      newState.linkTitle = (entityRange && entityRange.text) ||
        getCurrentSelectionText(editorState);
    }
    this.setState(newState);
  };

  _updateLinkTitle: Function = (event: Object): void => {
    this.setState({
      linkTitle: event.target.value,
    });
  };

  _updateLinkTarget: Function = (event: Object): void => {
    this.setState({
      linkTarget: event.target.value,
    });
  };

  _addLink: Function = (): void => {
    const { editorState, onChange } = this.props;
    const { linkTitle, linkTarget, entity } = this.state;
    let selection = editorState.getSelection();

    if (entity) {
      const entityRange = getEntityRange(editorState, entity);
      selection = selection.merge({
        anchorOffset: entityRange.start,
        focusOffset: entityRange.end,
      });
    }

    const entityKey = Entity.create('LINK', 'MUTABLE', {
      title: linkTitle,
      url: linkTarget,
    });
    const contentState = Modifier.replaceText(
      editorState.getCurrentContent(),
      selection,
      `${linkTitle} `,
      editorState.getCurrentInlineStyle(),
      entityKey,
    );
    let newEditorState = EditorState.push(editorState, contentState, 'insert-characters');
    const offset = selection.getAnchorOffset() + linkTitle.length;
    let updatedSelection = editorState.getSelection().merge({
      anchorOffset: offset,
      focusOffset: offset + 1,
    });
    newEditorState = RichUtils.toggleLink(newEditorState, updatedSelection, null);
    updatedSelection = newEditorState.getSelection().merge({
      anchorOffset: offset + 1,
      focusOffset: offset + 1,
    });
    newEditorState = EditorState.acceptSelection(newEditorState, updatedSelection);
    onChange(newEditorState);
    this._toggleLinkModal();
  };

  _removeLink: Function = (): void => {
    const { editorState, onChange, entity } = this.props;
    let selection = editorState.getSelection();
    if (entity) {
      const entityRange = getEntityRange(editorState, entity);
      delete entityRange.text;
      selection = selection.merge(entityRange);
    }
    if (!selection.isCollapsed()) {
      onChange(RichUtils.toggleLink(editorState, selection, null));
    }
  };

  _stopPropagation:Function = (event) => {
    event.stopPropagation();
  };

  renderAddLinkModal() {
    const { linkTitle, linkTarget } = this.state;
    return (
      <div
        className={styles['link-modal']}
        onClick={this._stopPropagation}
        onMouseDown={this._stopPropagation}
        onMouseUp={this._stopPropagation}
      >
        <div className={styles['link-modal-dialog']}>
          <div className={styles['link-modal-content']}>
            <span className={styles.link_modal_label}>Link Title</span>
            <input
              className={styles.link_modal_input}
              onChange={this._updateLinkTitle}
              onBlur={this._updateLinkTitle}
              value={linkTitle}
            />
            <span className={styles.link_modal_label}>Link Target</span>
            <input
              className={styles.link_modal_input}
              onChange={this._updateLinkTarget}
              onBlur={this._updateLinkTarget}
              value={linkTarget}
            />
          </div>
          <div className={styles['link-modal-footer']}>
            <span className={styles.link_modal_buttonsection}>
              <button
                className={styles.link_modal_btn}
                onClick={this._addLink}
                disabled={!linkTarget || !linkTitle}
              >
                Add
              </button>
              <button
                className={styles.link_modal_btn}
                onClick={this._toggleLinkModal}
              >
                Cancel
              </button>
            </span>
          </div>
        </div>
      </div>
    );
  }

  render(): Object {
    const { showLinkModal } = this.state;
    const { entity } = this.props;
    return (
      <div className={styles.link_wrapper}>
        <Option
          value="unordered-list-item"
          onClick={this._toggleLinkModal}
        >
          <a>
            <img
              src={link}
              role="presentation"
            />
          </a>
        </Option>
        <Option
          disabled={!entity}
          value="ordered-list-item"
          onClick={this._removeLink}
        >
          <img
            src={unlink}
            role="presentation"
          />
        </Option>
        {showLinkModal ? this.renderAddLinkModal() : undefined}
      </div>
    );
  }
}
