import {
  forEach,
  isEmptyString,
} from '../Common';

/**
* Mapping object of block and corresponding html tag.
*/
const blockTypesMapping: Object = {
  'header-one': 'h1',
  'header-two': 'h2',
  'header-three': 'h3',
  'header-four': 'h4',
  'header-five': 'h5',
  'header-six': 'h6',
  'unordered-list-item': 'ul',
  'ordered-list-item': 'ol',
  blockquote: 'blockquote',
};

/**
* Function will return HTML block tag for a block.
*/
export function getBlockTag(type: string): string {
  return type && blockTypesMapping[type] || 'p';
}

/**
* Function will return HTML block tag for a block.
*/
export function getBlockStyle(data: Object): string {
  let styles = '';
  forEach(data, (key, value) => {
    styles += `${key}:${value};`;
  });
  return styles;
}

/**
* The function returns an array of sections in blocks,
* depending on which entity are applicable to the block sections.
*/
function getSections(entityRanges: Object, blockLength: number): Array<Object> {
  const ranges = [];
  let lastOffset = 0;
  entityRanges.forEach((r) => {
    if (r.offset > lastOffset) {
      ranges.push({
        start: lastOffset,
        end: r.offset - 1,
      });
    }
    ranges.push({
      start: r.offset,
      end: r.offset + r.length - 1,
      entityKey: r.key,
    });
    lastOffset = r.offset + r.length;
  });
  if (lastOffset < blockLength) {
    ranges.push({
      start: lastOffset,
      end: blockLength,
    });
  }
  return ranges;
}

/**
* Function to check if the block is an atomic entity block.
*/
function isAtomicEntityBlock(block) {
  if (block.entityRanges.length > 0 && isEmptyString(block.text)) {
    return true;
  }
  return false;
}

/**
* Function to get markup for atomic entity block.
*/
function getAtomicEntityMarkup(block, entityMap) {
  const entity = entityMap[block.entityRanges[0].key];
  const { src, alignment, width } = entity.data;
  if (entity.type === 'IMAGE') {
    return `<img src="${src}" style="float: ${alignment || 'none'}; width: ${width}; height: auto"/>`;
  }
  return undefined;
}

/**
* The function will return array of inline styles applicable to the block.
*/
function getStylesArrayForBlock(block: Object): Object {
  const { text, inlineStyleRanges } = block;
  const inlineStyles = {
    BOLD: new Array(text.length),
    ITALIC: new Array(text.length),
    UNDERLINE: new Array(text.length),
    COLOR: new Array(text.length),
    FONTSIZE: new Array(text.length),
    length: text.length,
  };
  if (inlineStyleRanges && inlineStyleRanges.length > 0) {
    inlineStyleRanges.forEach((range) => {
      const offset = range.offset;
      const length = offset + range.length;
      for (let i = offset; i < length; i++) {
        if (range.style.startsWith('rgb(')) {
          inlineStyles.COLOR[i] = range.style;
        } else if (range.style.startsWith('fontsize-')) {
          inlineStyles.FONTSIZE[i] = range.style.substring(9, range.style.length);
        } else if (inlineStyles[range.style]) {
          inlineStyles[range.style][i] = true;
        }
      }
    });
  }
  return inlineStyles;
}

/**
* The function will return inline style applicable at some offset within a block.
*/
function getStylesAtOffset(inlineStyles: Object, offset: number): Object {
  const styles = {};
  if (inlineStyles.COLOR[offset]) {
    styles.COLOR = inlineStyles.COLOR[offset];
  }
  if (inlineStyles.FONTSIZE[offset]) {
    styles.FONTSIZE = inlineStyles.FONTSIZE[offset];
  }
  if (inlineStyles.UNDERLINE[offset]) {
    styles.UNDERLINE = true;
  }
  if (inlineStyles.ITALIC[offset]) {
    styles.ITALIC = true;
  }
  if (inlineStyles.BOLD[offset]) {
    styles.BOLD = true;
  }
  return styles;
}

/**
* Function returns true if given styles applicable at an offset
* are also applicable at index before it.
*/
function sameStyleAsPrevious(
  inlineStyles: Object,
  styles: Array<string>,
  index: number
): any {
  let sameStyled = true;
  if (index > 0 && index < inlineStyles.length) {
    styles.forEach((style) => {
      sameStyled = sameStyled && inlineStyles[style][index] === inlineStyles[style][index - 1];
    });
  } else {
    sameStyled = false;
  }
  return sameStyled;
}

/**
* Function returns html for text depending on inline style tags applicable to it.
*/
function addInlineStyleMarkup(style: string, content: string): string {
  if (style === 'BOLD') {
    return `<strong>${content}</strong>`;
  } else if (style === 'ITALIC') {
    return `<em>${content}</em>`;
  } else if (style === 'UNDERLINE') {
    return `<ins>${content}</ins>`;
  }
  return content;
}

/**
* Function returns html for text depending on inline style tags applicable to it.
*/
function addStylePropertyMarkup(styles: string, content: string): string {
  if (styles.COLOR || styles.FONTSIZE) {
    let styleString = ' style="';
    if (styles.COLOR) {
      styleString += `color: ${styles.COLOR};`;
    }
    if (styles.FONTSIZE) {
      styleString += `font-size: ${styles.FONTSIZE};`;
    }
    styleString += '"';
    return `<span ${styleString}>${content}</span>`;
  }
  return content;
}

/**
* Function will return markup for Entity.
*/
function addEntityMarkup(entityMap, entityKey, text) {
  const entity = entityMap[entityKey];
  if (entity.type === 'LINK') {
    return `<a href="${entity.data.url}">${text}</a>`;
  }
  return text;
}

/**
* The function will return an array of section with same value of inline styles passed
* within a section for the block.
*/
function getInlineStyleSections(
  block: Object,
  styles: Array<string>,
  start: number,
  end: number
): Array<Object> {
  const styleSections = [];
  const { text } = block;
  if (text.length > 0) {
    const inlineStyles = getStylesArrayForBlock(block);
    let section;
    for (let i = start; i < end; i++) {
      if (i !== start && sameStyleAsPrevious(inlineStyles, styles, i)) {
        section.text.push(text[i]);
        section.end = i + 1;
      } else {
        section = {
          styles: getStylesAtOffset(inlineStyles, i),
          text: [text[i]],
          start: i,
          end: i + 1,
        };
        styleSections.push(section);
      }
    }
  }
  return styleSections;
}

function getSectionText(text: Array<string>): string {
  if (text && text.length > 0) {
    const chars = text.map((ch) => {
      switch (ch) {
        case '\n':
          return '<br>\n';
        case '&':
          return '&amp;';
        case '<':
          return '&lt;';
        case '>':
          return '&gt;';
        default:
          return ch;
      }
    });
    return chars.join('');
  }
  return '';
}

function trimLeadingZeros(sectionText: string): string {
  let replacedText = sectionText;
  for (let i = 0; i < replacedText.length; i++) {
    if (sectionText[i] === ' ') {
      replacedText = replacedText.replace(' ', '&nbsp;');
    } else {
      break;
    }
  }
  return replacedText;
}

function trimTrailingZeros(sectionText: string): string {
  let replacedText = sectionText;
  for (let i = replacedText.length - 1; i >= 0; i--) {
    if (replacedText[i] === ' ') {
      replacedText = `${replacedText.substring(0, i)}&nbsp;${replacedText.substring(i + 1)}`;
    } else {
      break;
    }
  }
  return replacedText;
}

/**
* Function will return the content of block preserving the inline styles and
* special characters like newlines or blank spaces.
*/
export function getBlockContent(block: Object, entityMap: Object): string {
  const blockContent = [];
  const sections = getSections(block.entityRanges, block.text.length);
  sections.forEach((section, index) => {
    const styleSections = getInlineStyleSections(
      block, ['COLOR', 'FONTSIZE'], section.start, section.end
    );
    const sectionContent = [];
    styleSections.forEach((styleSection) => {
      const styleTagSections = getInlineStyleSections(
        block, ['BOLD', 'ITALIC', 'UNDERLINE'], styleSection.start, styleSection.end
      );
      let styleTagSectionText = '';
      styleTagSections.forEach((styleTagSection) => {
        let text = getSectionText(styleTagSection.text, 1);
        forEach(styleTagSection.styles, (style, value) => {
          text = addInlineStyleMarkup(style, text, value);
        });
        styleTagSectionText += text;
      });
      if (styleSection.styles) {
        styleTagSectionText = addStylePropertyMarkup(styleSection.styles, styleTagSectionText);
      }
      sectionContent.push(styleTagSectionText);
    });
    let sectionText = sectionContent.join('');
    if (section.entityKey !== undefined && section.entityKey !== null) {
      sectionText = addEntityMarkup(entityMap, section.entityKey, sectionText);
    }
    if (index === 0) {
      sectionText = trimLeadingZeros(sectionText);
    }
    if (index === sections.length - 1) {
      sectionText = trimTrailingZeros(sectionText);
    }
    blockContent.push(sectionText);
  });
  return blockContent.join('');
}

/**
* Function will return html for the block.
*/
export function getBlockHtml(block: Object, entityMap: Object):string {
  const blockHtml = [];
  if (isAtomicEntityBlock(block)) {
    const content = getAtomicEntityMarkup(block, entityMap);
    if (content) {
      blockHtml.push(getAtomicEntityMarkup(block, entityMap));
    }
  } else {
    const blockTag = getBlockTag(block.type);
    const blockStyle = getBlockStyle(block.data);
    blockHtml.push(`<${blockTag}`);
    if (blockStyle) {
      blockHtml.push(` style="${blockStyle}"`);
    }
    blockHtml.push('>');
    blockHtml.push(getBlockContent(block, entityMap));
    blockHtml.push(`</${blockTag}>\n`);
  }
  return blockHtml.join('');
}
