/* @flow */

import {
  EditorState,
  RichUtils,
  Modifier,
  ContentBlock,
} from 'draft-js';
import { OrderedMap, List } from 'immutable';

/**
* Function returns collection of currently selected blocks.
*/
export function getSelectedBlocks(editorState: EditorState): OrderedMap {
  const selectionState = editorState.getSelection();
  const contentState = editorState.getCurrentContent();
  const startKey = selectionState.getStartKey();
  const endKey = selectionState.getEndKey();
  const blockMap = contentState.getBlockMap();
  return blockMap
    .toSeq()
    .skipUntil((_, k) => k === startKey)
    .takeUntil((_, k) => k === endKey)
    .concat([[endKey, blockMap.get(endKey)]]);
}

export function getCurrentBlock(editorState: EditorState): ContentBlock {
  return getSelectedBlocks(editorState).toList().get(0);
}
/**
* Function returns collection of currently selected blocks.
*/
export function getBlocks(editorState: EditorState): List {
  return editorState.getCurrentContent().getBlockMap().toList();
}

/**
* Function returns first selected block.
*/
export function getFirstSelectedBlock(editorState: EditorState): OrderedMap {
  const selectionState = editorState.getSelection();
  const contentState = editorState.getCurrentContent();
  const startKey = selectionState.getStartKey();
  const blockMap = contentState.getBlockMap();
  const blockList = blockMap
    .toSeq()
    .skipUntil((_, k) => k === startKey)
    .toList();
  return blockList && blockList.size > 0 && blockList.get(0);
}

/**
* If all currently selected blocks are of same type the function will return their type,
* Else it will return empty string.
*/
export function getCurrentBlockType(editorState: EditorState): string {
  const blocks = getSelectedBlocks(editorState).toList();
  const hasMultipleBlockTypes = blocks.some((block) => block.type !== blocks.get(0).type);
  if (!hasMultipleBlockTypes) {
    return blocks.get(0).type;
  }
  return 'unstyled';
}

/**
* Function will change block style to unstyled for selected blocks.
*/
export function removeBlockStyle(editorState: EditorState): EditorState {
  const newContentState = RichUtils.tryToRemoveBlockStyle(editorState);
  return EditorState.push(editorState, newContentState, 'change-block-type');
}

/**
* Function will inset a new unstyled block.
*/
export function insertNewUnstyledBlock(editorState: EditorState): EditorState {
  const newContentState = Modifier.splitBlock(
    editorState.getCurrentContent(),
    editorState.getSelection()
  );
  const newEditorState = EditorState.push(editorState, newContentState, 'split-block');
  return removeBlockStyle(newEditorState);
}

/**
* Function will return currently selected text in the editor.
*/
export function getCurrentSelectionText(editorState: EditorState): string {
  let selectedText = '';
  const currentSelection = editorState.getSelection();
  let start = currentSelection.getAnchorOffset();
  let end = currentSelection.getFocusOffset();
  const selectedBlocks = getSelectedBlocks(editorState).toList();
  if (selectedBlocks.size > 0) {
    if (currentSelection.getIsBackward()) {
      const temp = start;
      start = end;
      end = temp;
    }
    for (let i = 0; i < selectedBlocks.size; i++) {
      const blockStart = i === 0 ? start : 0;
      const blockEnd =
        i === (selectedBlocks.size - 1) ? end : selectedBlocks.get(i).getText().length;
      selectedText += selectedBlocks.get(i).getText().slice(blockStart, blockEnd);
    }
  }
  return selectedText;
}

/**
* Function will handle followind keyPress scenario:
* case Shift+Enter, select not collapsed ->
*   selected text will be removed and line break will be inserted.
*/
export function addNewLineRemovingSelection(editorState: EditorState): EditorState {
  const content = editorState.getCurrentContent();
  const selection = editorState.getSelection();
  let newContent = Modifier.removeRange(content, selection, 'forward');
  const fragment = newContent.getSelectionAfter();
  const block = newContent.getBlockForKey(fragment.getStartKey());
  newContent = Modifier.insertText(
    newContent,
    fragment,
    '\n',
    block.getInlineStyleAt(fragment.getStartOffset()),
    null,
  );
  return EditorState.push(editorState, newContent, 'insert-fragment');
}

export function alignmentBlockStyleFn(block: Object): string {
  const blockAlignment = block.getData() && block.getData().get('text-align');
  if (blockAlignment) {
    return `${blockAlignment}-aligned-block`;
  }
  return '';
}
