/* @flow */

import {
  Entity,
  Modifier,
  RichUtils,
  EditorState,
  ContentBlock,
} from 'draft-js';
import { merge } from '../Common';
import { getSelectedBlocks, getCurrentBlock } from './Block';

/**
* Function returns color applicable at a offset.
*/
function getColorAtOffset(block: ContentBlock, offset: number): any {
  const styles = block.getInlineStyleAt(offset).toList();
  const colorStyle = styles.filter((style) => style.startsWith('rgb('));
  if (colorStyle && colorStyle.size > 0) {
    return colorStyle.get(0);
  }
  return undefined;
}

/**
* Function returns size at a offset.
*/
function getFontSizeAtOffset(block: ContentBlock, offset: number): any {
  const styles = block.getInlineStyleAt(offset).toList();
  const fontStyle = styles.filter((style) => style.startsWith('fontsize-'));
  if (fontStyle && fontStyle.size > 0) {
    return fontStyle.get(0);
  }
  return undefined;
}

/**
* Function returns an object of inline styles currently applicable:
* {
*   BOLD: true,
*   ITALIC: true,
*   UNDERLINE: true,
*   COLOR: rgb(X,X,X),
*   FONTSIZE: XX,
* }
* Following rules are applicable:
* - styles are all false if editor is not focused
* - if if focus is at beginning of the block and selection is collapsed
*     styles of first character in block is returned.
* - if focus id anywhere inside the block and selection is collapsed
*     style of a character before focus is returned.
*/
export function getInlineStylesOfSelection(editorState: EditorState): EditorState {
  const currentSelection = editorState.getSelection();
  const start = currentSelection.getStartOffset();
  const end = currentSelection.getEndOffset();
  const selectedBlocks = getSelectedBlocks(editorState).toList();
  if (selectedBlocks.size > 0) {
    const inlineStyles = {
      BOLD: true,
      ITALIC: true,
      UNDERLINE: true,
      COLOR: undefined,
      FONTSIZE: undefined,
    };
    for (let i = 0; i < selectedBlocks.size; i++) {
      let blockStart = i === 0 ? start : 0;
      let blockEnd =
        i === (selectedBlocks.size - 1) ? end : selectedBlocks.get(i).getText().length;
      if (blockStart === blockEnd && blockStart === 0) {
        blockStart = 1;
        blockEnd = 2;
      } else if (blockStart === blockEnd) {
        blockStart -= 1;
      }
      for (let j = blockStart; j < blockEnd; j++) {
        const inlineStylesAtOffset = selectedBlocks.get(i).getInlineStyleAt(j);
        inlineStyles.BOLD = inlineStyles.BOLD && inlineStylesAtOffset.get('BOLD') === 'BOLD';
        inlineStyles.ITALIC =
          inlineStyles.ITALIC && inlineStylesAtOffset.get('ITALIC') === 'ITALIC';
        inlineStyles.UNDERLINE =
          inlineStyles.UNDERLINE && inlineStylesAtOffset.get('UNDERLINE') === 'UNDERLINE';
        const colorAtOffset = getColorAtOffset(selectedBlocks.get(i), j);
        if (j === blockStart) {
          inlineStyles.COLOR = colorAtOffset;
        } else if (inlineStyles.COLOR && inlineStyles.COLOR !== colorAtOffset) {
          inlineStyles.COLOR = undefined;
        }
        const fontSizeAtOffset = getFontSizeAtOffset(selectedBlocks.get(i), j);
        if (j === blockStart) {
          inlineStyles.FONTSIZE = fontSizeAtOffset;
        } else if (inlineStyles.FONTSIZE && inlineStyles.FONTSIZE !== fontSizeAtOffset) {
          inlineStyles.FONTSIZE = undefined;
        }
      }
    }
    return inlineStyles;
  }
  return {
    BOLD: false,
    ITALIC: false,
    UNDERLINE: false,
    COLOR: undefined,
    FONTSIZE: undefined,
  };
}

/**
* Array of colors supported for custom inline styles.
*/
export const colors = ['rgb(97,189,109)', 'rgb(26,188,156)', 'rgb(84,172,210)', 'rgb(44,130,201)',
    'rgb(147,101,184)', 'rgb(71,85,119)', 'rgb(204,204,204)', 'rgb(65,168,95)', 'rgb(0,168,133)',
    'rgb(61,142,185)', 'rgb(41,105,176)', 'rgb(85,57,130)', 'rgb(40,50,78)', 'rgb(0,0,0)',
    'rgb(247,218,100)', 'rgb(251,160,38)', 'rgb(235,107,86)', 'rgb(226,80,65)', 'rgb(163,143,132)',
    'rgb(239,239,239)', 'rgb(255,255,255)', 'rgb(250,197,28)', 'rgb(243,121,52)', 'rgb(209,72,65)',
    'rgb(184,49,47)', 'rgb(124,112,107)', 'rgb(209,213,216)'];

/**
* Array of font-sizes supported for custom inline styles.
*/
export const fontSizes = [8, 9, 10, 11, 12, 14, 18, 24, 30, 36, 48, 60, 72, 96];

/**
* Collection of all custom inline styles.
* Structue of entity in map is key -> inline style.
* For color: 'rgb(97,189,109)': { color: 'rgb(97,189,109)'}
* For fontSize: '20': { fontSize: 20 }
*/
export const customInlineStylesMap = {
  color: {},
  fontSize: {},
};
colors.forEach((color) => {
  customInlineStylesMap.color[color] = {
    color,
  };
});
fontSizes.forEach((size) => {
  customInlineStylesMap.fontSize[`fontsize-${size}`] = {
    fontSize: size,
  };
});

/**
* Combined map of all custon inline styles used to initialize editor.
*/
export const customStyleMap = merge(customInlineStylesMap.color, customInlineStylesMap.fontSize);

/**
* Function to toggle a custom inline style in current selection current selection.
*/
export function toggleInlineStyle(
  editorState: EditorState,
  styleType: string,
  style: string
): EditorState {
  const selection = editorState.getSelection();
  const nextContentState = Object.keys(customInlineStylesMap[styleType])
    .reduce((contentState, s) => Modifier.removeInlineStyle(contentState, selection, s),
      editorState.getCurrentContent());
  let nextEditorState = EditorState.push(
    editorState,
    nextContentState,
    'change-inline-style'
  );
  const currentStyle = editorState.getCurrentInlineStyle();
  if (selection.isCollapsed()) {
    nextEditorState = currentStyle
      .reduce((state, s) => RichUtils.toggleInlineStyle(state, s),
      nextEditorState);
  }
  if (!currentStyle.has(style)) {
    nextEditorState = RichUtils.toggleInlineStyle(
      nextEditorState,
      style
    );
  }
  return nextEditorState;
}

/**
* This function will return the entity applicable to whole of current selection.
* An entity can not span multiple blocks.
*/
export function getEntityOfSelection(editorState: EditorState): Entity {
  let entity;
  const selection = editorState.getSelection();
  let start = selection.getStartOffset();
  let end = selection.getEndOffset();
  if (start === end && start === 0) {
    end = 1;
  } else if (start === end) {
    start -= 1;
  }
  const block = getCurrentBlock(editorState);

  for (let i = start; i < end; i++) {
    const currentEntity = block.getEntityAt(i);
    if (!currentEntity) {
      entity = undefined;
      break;
    }
    if (i === start) {
      entity = currentEntity;
    } else {
      if (entity !== currentEntity) {
        entity = undefined;
        break;
      }
    }
  }
  return entity;
}

/**
* The function returns the range of given entity inside the block.
* {
*   anchorOffset: undefined,
*   focusOffset: undefined,
*   text: undefined,
* }
*/
export function getEntityRange(editorState: EditorState, entityKey: string): any {
  const block = getCurrentBlock(editorState);
  let entityRange;
  block.findEntityRanges(
    (value) => value.get('entity') === entityKey,
    (start, end) => {
      entityRange = {
        start,
        end,
        text: block.get('text').slice(start, end),
      };
    }
  );
  return entityRange;
}
